{%block head%}
    <head>
        <meta charset="UTF-8">
    {%block head_js%}
        <script src="/static/js/jquery.min.js"></script>
        <script src="/static/js/bootstrap.min.js"></script>
        <script src="/static/js/popper.min.js"></script>
        <script src="/static/js/d3.js"></script>
        <script defer src="/static/js/all.js"></script>
        <script async defer src="https://buttons.github.io/buttons.js"></script>
    {%endblock%}
    {%block head_css%}
    <link rel="stylesheet" href="/static/css/bootstrap.css">
        {%endblock%}
    {%block title%}
        <title>Langhuan</title>
    {%endblock%}
    </head>
{%endblock%}

{%block content%}
{%endblock%}

{%block tail_js%}
{%endblock%}